#include "psocgpioint.h"
#include "lsm303dlm.h"
#include "types_n_macros.h"
//-----------------------------------------------------------------------
//
// IMPORTANT!!! 
// SDATA must be set in Pull-up mode to act as a bidirectional port. SDATA must be set to 1 to enable input mode!
// SCLK is set to output. 
//
//-----------------------------------------------------------------------
// I2C bus base addresses

#define I2C_ACC_ADDR_W 0x32
#define I2C_ACC_ADDR_R 0x33

#define I2C_MAG_ADDR_W 0x3C
#define I2C_MAG_ADDR_R 0x3D

//-----------------------------------------------------------------------
// I2C software implementation for LSM303
//-----------------------------------------------------------------------

void delay_us( void )
{
//unsigned char i;
// Ugly active waiting to meet I2C timings 
//for (i = 0; i < TIMING_DELAY; i++){;}	
//DELAY_12CYCLES
DELAY_4CYCLES 

}
//-----------------------------------------------------------------------
void I2C_Start( void )
{
GPIO_HIGH(SDATA);
GPIO_HIGH(SCLK);

delay_us();
GPIO_LOW(SDATA);
delay_us();
GPIO_LOW(SCLK);
delay_us();

}
//-----------------------------------------------------------------------
void I2C_RepeatStart( void )
{
//unsigned char i; 

GPIO_HIGH(SDATA);
delay_us();
GPIO_HIGH(SCLK);

delay_us();
GPIO_LOW(SDATA);
delay_us();
GPIO_LOW(SCLK);
delay_us();

}
//-----------------------------------------------------------------------
void I2C_Stop( void )
{
GPIO_LOW(SDATA);
delay_us();

GPIO_HIGH(SCLK);
delay_us();

GPIO_HIGH(SDATA);
delay_us();
}
//-----------------------------------------------------------------------
void I2C_TxByte( unsigned char data )
{
unsigned char i;
unsigned char mask = 0x80; // MSB first!

for (i = 0; i < 8; i++)
{
	if (data & mask)
	{
	 	GPIO_HIGH(SDATA);
	}
	else 
	{
		GPIO_LOW(SDATA);
	}
	mask >>= 0x01;
	
	//delay_us();

	GPIO_HIGH(SCLK);
	delay_us();
	delay_us();

	GPIO_LOW(SCLK);
	delay_us(); 
}

}
//-----------------------------------------------------------------------
unsigned char I2C_RxByte( void )
{

unsigned char t, data = 0x00;
unsigned char i;

GPIO_HIGH(SDATA); // To set the port in pseudo input

PRT2DM0 &= 0xbf;

for (i = 0; i < 8; i++)
{
	delay_us();

	GPIO_HIGH(SCLK);
	// Latch data bit... MSB first!
	delay_us();
	data <<= 0x01;
	if (SDATA_Data_ADDR & SDATA_MASK) data |= 0x01; 
	delay_us();
	GPIO_LOW(SCLK);
	delay_us(); 
}

PRT2DM0 |= 0x40;
 
return data;
}
//-----------------------------------------------------------------------
void I2C_SlaveAck( void )
{
// Slave acknowledge == Set SDATA as input and generate one clock. Slave sets SDATA to low.
// SDATA Check not implemented. 

GPIO_HIGH(SDATA); // To set the port in pseudo input

delay_us();

GPIO_HIGH(SCLK);
delay_us();
delay_us();

GPIO_LOW(SCLK);
delay_us(); 

}
//-----------------------------------------------------------------------
void I2C_MasterAck( void )
{

GPIO_LOW(SDATA);

delay_us();

GPIO_HIGH(SCLK);
delay_us();
delay_us();

GPIO_LOW(SCLK);
delay_us(); 
}
//-----------------------------------------------------------------------
void I2C_MasterNack( void )
{
GPIO_HIGH(SDATA); 

delay_us();

GPIO_HIGH(SCLK);
delay_us();
delay_us();

GPIO_LOW(SCLK);
delay_us(); 
}
//-----------------------------------------------------------------------
void LSM303_Start( void )
{
// Gives a I2C Stop for init...
GPIO_HIGH(SCLK);
GPIO_HIGH(SDATA);

// The acceleration sensor is in Power Down mode at reset!
// -> CTRL_REG1_A must be written with proper value to stratup the sensor!	
//LSM303_TxReg_Acc(0x20, 0x3f); // PM = 1 + all axes enabled + freq = 1kHz --- NOTE: 1kHz and 400Hz do not seem to work!!!

LSM303_TxReg_Acc(0x20, 0x2f); // PM = 1 + all axes enabled + freq = 100Hz
LSM303_TxReg_Acc(0x21, 0x00); // HF filter off
LSM303_TxReg_Acc(0x23, 0x80); // BUD to one to prevent changes between LSB and MSB readings...

// The magnetic sensor is in sleep mode at reset!
// -> MR_REG_M must be written with proper value to stratup the sensor!

LSM303_TxReg_Mag(0x00, 0x18); // Set the max bandwidth!!!
LSM303_TxReg_Mag(0x01, 0x60); // Set device gain -> +/- 2.5G
LSM303_TxReg_Mag(0x02, 0x00); // Mode set to continuous mode
}
//-----------------------------------------------------------------------
void LSM303_Stop( void )
{
// Set acceleration sensor in Power down + disable all axes
LSM303_TxReg_Acc(0x20, 0x00);

// Set magnetic sensor to sleep mode
LSM303_TxReg_Mag(0x02, 0x03);
}
//-----------------------------------------------------------------------
// Acceleration sensor access functions
//-----------------------------------------------------------------------
void LSM303_TxReg_Acc(unsigned char address, unsigned char data)
{

// Check that the target address is not reserved!!!
// Writing to these registers could damage the device!!!
//if (address < 0x20) return;
//if ((address > 0x2D) && (address < 0x30)) return;
//if (address > 0x37) return;

// I2C communication
I2C_Start();

// Write device address
I2C_TxByte(I2C_ACC_ADDR_W);
I2C_SlaveAck();

// Write register address
I2C_TxByte(address);
I2C_SlaveAck();

// Write register value
I2C_TxByte(data);
I2C_SlaveAck();

I2C_Stop();
}
//-----------------------------------------------------------------------
unsigned char LSM303_RxReg_Acc(unsigned char address)
{
unsigned char d = 0x00;

// Check that the target address is not reserved!!!
// Writing to these registers could damage the device!!!
//if (address < 0x20) return 0;
//if ((address > 0x2D) && (address < 0x30)) return 0;
//if (address > 0x37) return 0;

// I2C communication
I2C_Start();

// Write device address
I2C_TxByte(I2C_ACC_ADDR_W);
I2C_SlaveAck();

// Write register address
I2C_TxByte(address);
I2C_SlaveAck();

I2C_RepeatStart();

// Write device address in read mode
I2C_TxByte(I2C_ACC_ADDR_R);
I2C_SlaveAck();

// Read register value
d = I2C_RxByte();
I2C_MasterNack();

I2C_Stop();

return d;
}
//-----------------------------------------------------------------------
unsigned char LSM303_Check_DRDY_Acc( void )
{
	// Read status register from the LSM303 Acc sensor
	unsigned char status = LSM303_RxReg_Acc(0x27); // STATUS_REG_A
	
	// Check flag ZXYDA to see if a whole set is ready...
	return (status & 0x08);
}
//-----------------------------------------------------------------------
// Magnetic sensor access functions
//-----------------------------------------------------------------------
void LSM303_TxReg_Mag(unsigned char address, unsigned char data)
{
// Check that the target address is not reserved!!!
// Writing to these registers could damage the device!!!
//if (address > 0x0C) return;

// I2C communication
I2C_Start();

// Write device address
I2C_TxByte(I2C_MAG_ADDR_W);
I2C_SlaveAck();

// Write register address
I2C_TxByte(address);
I2C_SlaveAck();

// Write register value
I2C_TxByte(data);
I2C_SlaveAck();

I2C_Stop();
}
//-----------------------------------------------------------------------
unsigned char LSM303_RxReg_Mag(unsigned char address)
{
unsigned char d = 0x00;

// Check that the target address is not reserved!!!
// Writing to these registers could damage the device!!!
//if (address > 0x0C) return 0;

// I2C communication
I2C_Start();

// Write device address
I2C_TxByte(I2C_MAG_ADDR_W);
I2C_SlaveAck();

// Write register address
I2C_TxByte(address);
I2C_SlaveAck();

I2C_RepeatStart();

// Write device address in read mode
I2C_TxByte(I2C_MAG_ADDR_R);
I2C_SlaveAck();

// Read register value
d = I2C_RxByte();
I2C_MasterNack();

I2C_Stop();

return d;
}
//-----------------------------------------------------------------------
unsigned char LSM303_Check_DRDY_Mag()
{
// Check MDRDY pin. Note that it could also be implemented by reading an internal register...
return (MAG_DRDY_Data_ADDR & MAG_DRDY_MASK); 
}

//-----------------------------------------------------------------------
// Conversion tables for magnetic sensor (+/-2.5 Gauss on 12bits)
//-----------------------------------------------------------------------

const int Magnetic_X_Y_Table_MSB[8] =
{
0,
4031,
8063,
12094,
16126,
20157,
24189,
28220
};

const int Magnetic_X_Y_Table_LSB[256] =
{
0,
16,
31,
47,
63,
79,
94,
110,
126,
142,
157,
173,
189,
205,
220,
236,
252,
268,
283,
299,
315,
331,
346,
362,
378,
394,
409,
425,
441,
457,
472,
488,
504,
520,
535,
551,
567,
583,
598,
614,
630,
646,
661,
677,
693,
709,
724,
740,
756,
772,
787,
803,
819,
835,
850,
866,
882,
898,
913,
929,
945,
961,
976,
992,
1008,
1024,
1039,
1055,
1071,
1087,
1102,
1118,
1134,
1150,
1165,
1181,
1197,
1213,
1228,
1244,
1260,
1276,
1291,
1307,
1323,
1339,
1354,
1370,
1386,
1402,
1417,
1433,
1449,
1465,
1480,
1496,
1512,
1528,
1543,
1559,
1575,
1591,
1606,
1622,
1638,
1654,
1669,
1685,
1701,
1717,
1732,
1748,
1764,
1780,
1795,
1811,
1827,
1843,
1858,
1874,
1890,
1906,
1921,
1937,
1953,
1969,
1984,
2000,
2016,
2031,
2047,
2063,
2079,
2094,
2110,
2126,
2142,
2157,
2173,
2189,
2205,
2220,
2236,
2252,
2268,
2283,
2299,
2315,
2331,
2346,
2362,
2378,
2394,
2409,
2425,
2441,
2457,
2472,
2488,
2504,
2520,
2535,
2551,
2567,
2583,
2598,
2614,
2630,
2646,
2661,
2677,
2693,
2709,
2724,
2740,
2756,
2772,
2787,
2803,
2819,
2835,
2850,
2866,
2882,
2898,
2913,
2929,
2945,
2961,
2976,
2992,
3008,
3024,
3039,
3055,
3071,
3087,
3102,
3118,
3134,
3150,
3165,
3181,
3197,
3213,
3228,
3244,
3260,
3276,
3291,
3307,
3323,
3339,
3354,
3370,
3386,
3402,
3417,
3433,
3449,
3465,
3480,
3496,
3512,
3528,
3543,
3559,
3575,
3591,
3606,
3622,
3638,
3654,
3669,
3685,
3701,
3717,
3732,
3748,
3764,
3780,
3795,
3811,
3827,
3843,
3858,
3874,
3890,
3906,
3921,
3937,
3953,
3969,
3984,
4000,
4016
};

const int Magnetic_Z_Table_MSB[8] =
{
0,
4491,
8982,
13474,
17965,
22456,
26947,
31439
};

const int Magnetic_Z_Table_LSB[256] =
{
0,
18,
35,
53,
70,
88,
105,
123,
140,
158,
175,
193,
211,
228,
246,
263,
281,
298,
316,
333,
351,
368,
386,
404,
421,
439,
456,
474,
491,
509,
526,
544,
561,
579,
596,
614,
632,
649,
667,
684,
702,
719,
737,
754,
772,
789,
807,
825,
842,
860,
877,
895,
912,
930,
947,
965,
982,
1000,
1018,
1035,
1053,
1070,
1088,
1105,
1123,
1140,
1158,
1175,
1193,
1211,
1228,
1246,
1263,
1281,
1298,
1316,
1333,
1351,
1368,
1386,
1404,
1421,
1439,
1456,
1474,
1491,
1509,
1526,
1544,
1561,
1579,
1596,
1614,
1632,
1649,
1667,
1684,
1702,
1719,
1737,
1754,
1772,
1789,
1807,
1825,
1842,
1860,
1877,
1895,
1912,
1930,
1947,
1965,
1982,
2000,
2018,
2035,
2053,
2070,
2088,
2105,
2123,
2140,
2158,
2175,
2193,
2211,
2228,
2246,
2263,
2281,
2298,
2316,
2333,
2351,
2368,
2386,
2404,
2421,
2439,
2456,
2474,
2491,
2509,
2526,
2544,
2561,
2579,
2596,
2614,
2632,
2649,
2667,
2684,
2702,
2719,
2737,
2754,
2772,
2789,
2807,
2825,
2842,
2860,
2877,
2895,
2912,
2930,
2947,
2965,
2982,
3000,
3018,
3035,
3053,
3070,
3088,
3105,
3123,
3140,
3158,
3175,
3193,
3211,
3228,
3246,
3263,
3281,
3298,
3316,
3333,
3351,
3368,
3386,
3404,
3421,
3439,
3456,
3474,
3491,
3509,
3526,
3544,
3561,
3579,
3596,
3614,
3632,
3649,
3667,
3684,
3702,
3719,
3737,
3754,
3772,
3789,
3807,
3825,
3842,
3860,
3877,
3895,
3912,
3930,
3947,
3965,
3982,
4000,
4018,
4035,
4053,
4070,
4088,
4105,
4123,
4140,
4158,
4175,
4193,
4211,
4228,
4246,
4263,
4281,
4298,
4316,
4333,
4351,
4368,
4386,
4404,
4421,
4439,
4456,
4474
};

//-----------------------------------------------------------------------
// Conversion tables for magnetic sensor (+/-1.3 Gauss on 12bits)
//-----------------------------------------------------------------------
/*
const int Magnetic_X_Y_Table_MSB[8] =
{
0,
2427,
4853,
7280,
9706,
12133,
14559,
16986
};

const int Magnetic_X_Y_Table_LSB[256] =
{
0,
9,
19,
28,
38,
47,
57,
66,
76,
85,
95,
104,
114,
123,
133,
142,
152,
161,
171,
180,
190,
199,
209,
218,
227,
237,
246,
256,
265,
275,
284,
294,
303,
313,
322,
332,
341,
351,
360,
370,
379,
389,
398,
408,
417,
427,
436,
445,
455,
464,
474,
483,
493,
502,
512,
521,
531,
540,
550,
559,
569,
578,
588,
597,
607,
616,
626,
635,
645,
654,
664,
673,
682,
692,
701,
711,
720,
730,
739,
749,
758,
768,
777,
787,
796,
806,
815,
825,
834,
844,
853,
863,
872,
882,
891,
900,
910,
919,
929,
938,
948,
957,
967,
976,
986,
995,
1005,
1014,
1024,
1033,
1043,
1052,
1062,
1071,
1081,
1090,
1100,
1109,
1118,
1128,
1137,
1147,
1156,
1166,
1175,
1185,
1194,
1204,
1213,
1223,
1232,
1242,
1251,
1261,
1270,
1280,
1289,
1299,
1308,
1318,
1327,
1336,
1346,
1355,
1365,
1374,
1384,
1393,
1403,
1412,
1422,
1431,
1441,
1450,
1460,
1469,
1479,
1488,
1498,
1507,
1517,
1526,
1536,
1545,
1555,
1564,
1573,
1583,
1592,
1602,
1611,
1621,
1630,
1640,
1649,
1659,
1668,
1678,
1687,
1697,
1706,
1716,
1725,
1735,
1744,
1754,
1763,
1773,
1782,
1791,
1801,
1810,
1820,
1829,
1839,
1848,
1858,
1867,
1877,
1886,
1896,
1905,
1915,
1924,
1934,
1943,
1953,
1962,
1972,
1981,
1991,
2000,
2009,
2019,
2028,
2038,
2047,
2057,
2066,
2076,
2085,
2095,
2104,
2114,
2123,
2133,
2142,
2152,
2161,
2171,
2180,
2190,
2199,
2209,
2218,
2227,
2237,
2246,
2256,
2265,
2275,
2284,
2294,
2303,
2313,
2322,
2332,
2341,
2351,
2360,
2370,
2379,
2389,
2398,
2408,
2417
};

const int Magnetic_Z_Table_MSB[8] =
{
0,
2695,
5389,
8084,
10779,
13474,
16168,
18863
};

const int Magnetic_Z_Table_LSB[256] =
{
0,
11,
21,
32,
42,
53,
63,
74,
84,
95,
105,
116,
126,
137,
147,
158,
168,
179,
189,
200,
211,
221,
232,
242,
253,
263,
274,
284,
295,
305,
316,
326,
337,
347,
358,
368,
379,
389,
400,
411,
421,
432,
442,
453,
463,
474,
484,
495,
505,
516,
526,
537,
547,
558,
568,
579,
589,
600,
611,
621,
632,
642,
653,
663,
674,
684,
695,
705,
716,
726,
737,
747,
758,
768,
779,
789,
800,
811,
821,
832,
842,
853,
863,
874,
884,
895,
905,
916,
926,
937,
947,
958,
968,
979,
989,
1000,
1011,
1021,
1032,
1042,
1053,
1063,
1074,
1084,
1095,
1105,
1116,
1126,
1137,
1147,
1158,
1168,
1179,
1189,
1200,
1211,
1221,
1232,
1242,
1253,
1263,
1274,
1284,
1295,
1305,
1316,
1326,
1337,
1347,
1358,
1368,
1379,
1389,
1400,
1411,
1421,
1432,
1442,
1453,
1463,
1474,
1484,
1495,
1505,
1516,
1526,
1537,
1547,
1558,
1568,
1579,
1589,
1600,
1611,
1621,
1632,
1642,
1653,
1663,
1674,
1684,
1695,
1705,
1716,
1726,
1737,
1747,
1758,
1768,
1779,
1789,
1800,
1811,
1821,
1832,
1842,
1853,
1863,
1874,
1884,
1895,
1905,
1916,
1926,
1937,
1947,
1958,
1968,
1979,
1989,
2000,
2011,
2021,
2032,
2042,
2053,
2063,
2074,
2084,
2095,
2105,
2116,
2126,
2137,
2147,
2158,
2168,
2179,
2189,
2200,
2211,
2221,
2232,
2242,
2253,
2263,
2274,
2284,
2295,
2305,
2316,
2326,
2337,
2347,
2358,
2368,
2379,
2389,
2400,
2411,
2421,
2432,
2442,
2453,
2463,
2474,
2484,
2495,
2505,
2516,
2526,
2537,
2547,
2558,
2568,
2579,
2589,
2600,
2611,
2621,
2632,
2642,
2653,
2663,
2674,
2684
};
*/
//-----------------------------------------------------------------------------
// Scaling the Magnetic sensor value  
//-----------------------------------------------------------------------------
int MagSensorToPhysicalVal(int sensorVal, unsigned char Zaxis)
{
unsigned int val;
int physicalVal;

// Test if read value is negative or positive
if (sensorVal & 0x0800) 
{
	val = (unsigned int)((~sensorVal) + 1); //(sensorVal * -1); // * -1 could be optimized by (~val)+1 ... To be tested... 
}
else val = (unsigned int)sensorVal;

// Apply look table to scale absolute value...
if (Zaxis)
{
	physicalVal = Magnetic_Z_Table_MSB[(unsigned char)((val & 0x0f00) >> 8)] + Magnetic_Z_Table_LSB[(unsigned char)(val & 0x00ff)];
}
else
{
	physicalVal = Magnetic_X_Y_Table_MSB[(unsigned char)((val & 0x0f00) >> 8)] + Magnetic_X_Y_Table_LSB[(unsigned char)(val & 0x00ff)];
}

// If input was negative reset sign...
if (sensorVal & 0x0800) 
{
	//physicalVal *= -1; // * -1 could be optimized by (~val)+1 ... To be tested... 
	physicalVal = ((~physicalVal) + 1);
} 

return physicalVal; 
}