//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.h
//  Version: 1.0, 2007/03/08 
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Sensor specific functions and declarations
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
#include <m8c.h>
#include "parameters.h"

//-----------------------------------------------------------------------------
// Sensor parameters
// the first parameters (lowest indices) are identical for each sensor
// additional parameters may be defined here 
//-----------------------------------------------------------------------------
// actual used number of parameters
// --> see also #define MAX_PARAM_8 ... in parameters.h
#define NUM_PARAM_8		4	
#define NUM_PARAM_16	27	
#define NUM_PARAM_32	1
#define NUM_PARAM_168	8

// Assignment of Parameters
#define PAR8_REPORTMODE	0
#define PAR8_LEDMODE 1
#define PAR8_ACC_RANGE 2
#define PAR8_MAG_RANGE 3

#define PAR16_REPORTRATE_MS	0
#define PAR16_SAMPLERATE_MS	1
#define PAR16_SAMPLERATE_US	2
// PAR16 index 3 to 26 are used for customer calibration storage!!!

#define PAR32_DUMMY			0

#define PAR168_USRDEVNAME	0
#define PAR168_USRCHNAME_0	1
#define PAR168_USRCHNAME_1	2
#define PAR168_USRCHNAME_2	3
#define PAR168_USRCHNAME_3	4
#define PAR168_USRCHNAME_4	5
#define PAR168_USRCHNAME_5	6
#define PAR168_USRCHNAME_6	7




// factory defaults are defined in the source file of the sensor
extern	const BYTE			FactoryDefault_Param_8   [NUM_PARAM_8];
extern	const WORD			FactoryDefault_Param_16  [NUM_PARAM_16];
extern	const DWORD			FactoryDefault_Param_32  [NUM_PARAM_32];
extern	const tParam_168	FactoryDefault_Param_168 [NUM_PARAM_168];

//-----------------------------------------------------------------------------
// Type Definitions
//-----------------------------------------------------------------------------
typedef struct
{
	BYTE Ch0_LSB;
	BYTE Ch0_MSB;
	BYTE Ch1_LSB;
	BYTE Ch1_MSB;
	BYTE Ch2_LSB;
	BYTE Ch2_MSB;
	BYTE Ch3_LSB;
	BYTE Ch3_MSB;
	BYTE Ch4_LSB;
	BYTE Ch4_MSB;
	BYTE Ch5_LSB;
	BYTE Ch5_MSB;
	BYTE Ch6_LSB;
	BYTE Ch6_MSB;
	
} tReport;

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
extern tReport Report;
#define IN_REPORT_SIZE (sizeof(tReport))

extern BOOL	NewSample;

//-----------------------------------------------------------------------------
// Initializes the MS5534 
//-----------------------------------------------------------------------------
extern void Sensor_Start (void);

//-----------------------------------------------------------------------------
// Prepare Measurement for Suspend
//-----------------------------------------------------------------------------
extern void Sensor_Suspend(void);

//-----------------------------------------------------------------------------
// Send Sequence to Start Temperature Measurement
//-----------------------------------------------------------------------------
extern void Sensor_Resume(void);

//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the SCA100T sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
extern void Sensor_FSM_Measure(void);

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
extern BYTE Sensor_Specific_Feature(void);

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
extern void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes);

//-----------------------------------------------------------------------------
// debug function
//-----------------------------------------------------------------------------
extern void Sensor_Debug(void);

// end of file sensor.h
