//*****************************************************************************
//*****************************************************************************
//  FILENAME:   types_n_macros.h
//  Version: 1.0, 2007/03/20
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Definition of commonly used macros and data types
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//------------------------------------------------------------------------------
//*****************************************************************************
#include "psocgpioint.h"


//-----------------------------------------------------------------------------
// type definitions
//-----------------------------------------------------------------------------
typedef 
	struct sBigEndian
	{
		BYTE Hi;
		BYTE Lo;
	}; 
	struct sLittleEndian
	{
		BYTE Lo;
		BYTE Hi;
	}; 
	union uBigEndian
	{
		WORD w;
		struct sBigEndian BE;
		struct sLittleEndian LE;
	};


//-----------------------------------------------------------------------------
// macros for GPIO access
//-----------------------------------------------------------------------------

// set gpio level
#define GPIO_LOW(BIT)  BIT##_Data_ADDR &= ~BIT##_MASK;
#define GPIO_HIGH(BIT) BIT##_Data_ADDR |= BIT##_MASK;

// set GPIO drivemodes
// ==> see Cypress' Technical Reference Manual for more details about the modes
#define GPIO_DM_PULLDOWN(BIT)  	\
	BIT##_DriveMode_2_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_1_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_0_ADDR &= ~BIT##_MASK;

#define GPIO_DM_STRONG(BIT)  	\
	BIT##_DriveMode_2_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_1_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_0_ADDR |=  BIT##_MASK;

#define GPIO_DM_HIGHZ(BIT)  	\
	BIT##_DriveMode_2_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_1_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_0_ADDR &= ~BIT##_MASK;

#define GPIO_DM_PULLUP(BIT)  	\
	BIT##_DriveMode_2_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_1_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_0_ADDR |=  BIT##_MASK;

#define GPIO_DM_OPENSOURCE(BIT)  	\
	BIT##_DriveMode_2_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_1_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_0_ADDR &= ~BIT##_MASK;

#define GPIO_DM_STRONGSLOW(BIT)  	\
	BIT##_DriveMode_2_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_1_ADDR &= ~BIT##_MASK; \
	BIT##_DriveMode_0_ADDR |=  BIT##_MASK;

#define GPIO_DM_HIGHZANALOG(BIT)  	\
	BIT##_DriveMode_2_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_1_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_0_ADDR &= ~BIT##_MASK;

#define GPIO_DM_OPENDRAIN(BIT)  	\
	BIT##_DriveMode_2_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_1_ADDR |=  BIT##_MASK; \
	BIT##_DriveMode_0_ADDR |=  BIT##_MASK;

//-----------------------------------------------------------------------------
// macros for delay
//-----------------------------------------------------------------------------
#define DELAY_4CYCLES  asm("NOP");
#define DELAY_8CYCLES  asm("NOP \n NOP");
#define DELAY_12CYCLES asm("NOP \n NOP \n NOP");
#define DELAY_16CYCLES asm("NOP \n NOP \n NOP \n NOP");